require "spec_helper"

class Post
  def initialize(owner)
    @owner = owner
  end

  def owner?(user)
    @owner == user
  end
end

class OwnerCheck
  attr_reader :candidate, :resource

  def initialize(candidate, resource)
    @candidate = candidate
    @resource = resource
  end

  def success?
    resource.owner?(candidate)
  end
end

REPO = Permify::Repository.new

REPO.static(Post, :show)
REPO.static(Post, :create)

# :edit permission granted if either of its dependencies (:edit_all or
# :edit_own) is granted
REPO.any(Post, :edit, { Post => [:edit_all, :edit_own] })

REPO.static(Post, :edit_all)
REPO.dynamic(Post, :edit_own, [OwnerCheck])

class Admin
  def permissions
    Permify::Clearance.new(REPO).tap do |c|
      c.add Post, :show
      c.add Post, :create
      c.add Post, :edit_all
    end
  end
end

class User
  def permissions
    Permify::Clearance.new(REPO).tap do |c|
      c.add Post, :show
      c.add Post, :create
      c.add Post, :edit_own
    end
  end
end

describe Permify::Authorization, "integration" do
  let(:post) { Post.new(post_owner) }
  let(:post_owner) { user }
  let(:admin) { Admin.new }
  let(:user) { User.new }

  describe "admin" do
    let(:auth) { Permify::Authorization.new(admin) }

    describe "static create" do
      subject { auth.can?(:create, post)}
      it { should be_true }
    end

    describe "static show" do
      subject { auth.can?(:show, post)}
      it { should be_true }
    end

    describe "static edit" do
      subject { auth.can?(:edit, post)}
      it { should be_true }
    end
  end

  describe "user" do
    let(:auth) { Permify::Authorization.new(user) }

    describe "Post :edit_own" do
      subject { auth.can?(:edit_own, post)}

      context "when owner" do
        let(:post_owner) { user }
        it { should be_true }
      end

      context "when not owner" do
        let(:post_owner) { admin }
        it { should_not be_true }
      end
    end

    describe "Post :edit" do
      subject { auth.can?(:edit, post)}

      context "when owner" do
        let(:post_owner) { user }
        it { should be_true }
      end

      context "when not owner" do
        let(:post_owner) { admin }
        it { should_not be_true }
      end
    end
  end
end
